from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GL.ARB.framebuffer_object import *
from OpenGL.GL.EXT.framebuffer_object import *

import pygame
import Image
import sys,time

from ctypes import *
from math import *

def createAndCompileShader(type,source):
	shader=glCreateShader(type)
        glShaderSource(shader,source)
        glCompileShader(shader)

        # get "compile status" - glCompileShader will not fail with
        # an exception in case of syntax errors

        result=glGetShaderiv(shader,GL_COMPILE_STATUS)

        if (result!=1): # shader didn't compile
                raise Exception("Couldn't compile shader\nShader compilation Log:\n"+glGetShaderInfoLog(shader))
        return shader

def enableRenderHUD():
	glPushAttrib( GL_ENABLE_BIT )

	glDisable( GL_DEPTH_TEST )
	glDisable( GL_CULL_FACE )
	glDisable( GL_TEXTURE_2D )
	glDisable( GL_BLEND )
	
	glMatrixMode( GL_MODELVIEW )
	glPushMatrix()
	glLoadIdentity()
	glMatrixMode( GL_PROJECTION )
	glPushMatrix()
	glLoadIdentity()
	gluOrtho2D(-1,1,-1,1)

def disableRenderHUD():
	glMatrixMode( GL_PROJECTION )
	glPopMatrix()
	glMatrixMode( GL_MODELVIEW )
	glPopMatrix()

	glPopAttrib()		
	

class Shader:
    def __init__(self):
        print "Error: parameters needed"
        
    def __init__(self, vs_source, ps_source):
        
        vs_shader = createAndCompileShader(GL_VERTEX_SHADER,vs_source)
        ps_shader = createAndCompileShader(GL_FRAGMENT_SHADER,ps_source)

        self.program = glCreateProgram()
        glAttachShader(self.program,vs_shader)
        glAttachShader(self.program,ps_shader)
        glLinkProgram(self.program)

        self.enable()
     
    def setUniformFloat(self,name,value):
        loc = glGetUniformLocation(self.program,name);
        if (loc == -1):
		print "Error: "+name+" var not found in shader"
	else:
	        glUniform1f(loc,value);

    def enable(self):
            try:
                glUseProgram(self.program)
            except OpenGL.error.GLError:
                print glGetProgramInfoLog(self.program)
                raise

    def disable(self):
            glUseProgram(0)


class RenderTexture:
    def __init__(self,width,height,depth,filter=True):

	self.texture = glGenTextures( 1 )
	self.width = width
	self.height = height
	self.depth = depth

	glBindTexture( GL_TEXTURE_2D, self.texture );
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,	GL_REPEAT );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,	GL_REPEAT );
	if (filter):
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
	else:
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
		
	glTexImage2D(GL_TEXTURE_2D, 0,depth,width,height,0,depth,
		     GL_UNSIGNED_BYTE, None)

	self.fbo = glGenFramebuffers(1)

    def enable(self):
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, self.fbo)
	# In case of errors or suspect behaviour, try this:
	#    print "Framebuffer Status:"
	#    print glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
	
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
	GL_TEXTURE_2D, self.texture, 0)
	glPushAttrib(GL_VIEWPORT_BIT) # save viewport
	glViewport(0, 0, self.width, self.height) # set to fire effect dimensions

    def disable(self):
	glPopAttrib()
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0)

    def bind(self):
	glBindTexture( GL_TEXTURE_2D, self.texture )
	glEnable( GL_TEXTURE_2D )

    def unbind(self):
	glDisable( GL_TEXTURE_2D )

    def render(self):
	enableRenderHUD()
	self.bind()

	glBegin(GL_QUADS)
	glColor3f(1,1,1)
	glTexCoord2f(0,0)
	glVertex3f(-1,-1,0)
	glTexCoord2f(1,0)
	glVertex3f( 1,-1,0)
	glTexCoord2f(1,1)
	glVertex3f( 1, 1,0)
	glTexCoord2f(0,1)
	glVertex3f(-1, 1,0)
	glEnd()

	disableRenderHUD()

class Box:
    def __init__(self,color):

        self.cubelist = glGenLists(1);
        
        glNewList(self.cubelist,GL_COMPILE)
        glBegin(GL_QUADS)
        glColor3f(color[0],color[1],color[2])

        glNormal3f(0,0,-1)
        glVertex3f( -1, 1, -1)
        glVertex3f( 1, 1, -1)
        glVertex3f( 1, -1, -1)
        glVertex3f( -1, -1, -1)
        
        glNormal3f(0,0,1)
        glVertex3f( -1, -1, 1)
        glVertex3f( 1, -1, 1)
        glVertex3f( 1, 1, 1)
        glVertex3f( -1, 1, 1)

        glNormal3f(0,-1,0)
        glVertex3f( -1, -1, -1)
        glVertex3f( 1, -1, -1)
        glVertex3f( 1, -1, 1)
        glVertex3f( -1, -1, 1)

        glNormal3f(0,1,0)
        glVertex3f( -1, 1, 1)
        glVertex3f( 1, 1, 1)
        glVertex3f( 1, 1, -1)
        glVertex3f( -1, 1, -1)

        glNormal3f(-1,0,0)
        glVertex3f( -1, -1, 1)
        glVertex3f( -1, 1, 1)
        glVertex3f( -1, 1, -1)
        glVertex3f( -1, -1, -1)

        glNormal3f(1,0,0)
        glVertex3f( 1, -1, -1)
        glVertex3f( 1, 1, -1)
        glVertex3f( 1, 1, 1)
        glVertex3f( 1, -1, 1)
        
        glEnd()
        glEndList()

    def render(self):
        glCallList(self.cubelist)

# *************************************
def renderFPSbar(elapsed_time_in_ms):
	if (elapsed_time_in_ms == 0): elapsed_time = 1
	total = 0.01666 / (elapsed_time_in_ms * 0.001)

	enableRenderHUD()

	glLineWidth(4)
	glBegin(GL_LINES)
	glColor3f(0.5,0.5,0.7)
	glVertex3f(-1,-1,0)
	glVertex3f(-1 + 2 * total,-1,0)
	glEnd()

	disableRenderHUD()

def distance(a,b):
	v = [ b[0]-a[0], b[1]-a[1], b[2]-a[2] ]
	return sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2])

def jpg_file_write(name, number, data):
	im = Image.frombuffer("RGBA", (800,600), data, "raw", "RGBA", 0, 0)
	fnumber = "%05d" % number
	im.save(name + fnumber + ".jpg")
