
from OpenGL.GL import *
from OpenGL.GLU import *
import random
from math import * # trigonometry

import pygame, sys,os, time
from pygame.locals import * 

import Image

from GLTools import *


render_lines = False
rotate_cubes = False
use_blending = True
shape_sphere = False
pixelate = False
t=100
dist = 0.5
voxelsize = 4


#exit clean
def process_input(events):
    global render_lines,rotate_cubes,use_blending,voxelsize,shape_sphere,pixelate
    
    for event in events: 
        if event.type == QUIT:
            return False
        if event.type == pygame.KEYDOWN:
            if event.key == pygame.K_ESCAPE: 
                return False
            if event.key == pygame.K_1:
                render_lines = not render_lines
            if event.key == pygame.K_2:
                rotate_cubes = not rotate_cubes
            if event.key == pygame.K_3:
                use_blending = not use_blending
            if event.key == pygame.K_4:
                shape_sphere = not shape_sphere
            if event.key == pygame.K_5:
                pixelate = not pixelate
            if event.key == pygame.K_KP_PLUS:
                voxelsize+=1
            if event.key == pygame.K_KP_MINUS:
                voxelsize-=1
        else: 
            pass #print event
    return True

# get an OpenGL surface
pygame.init()
pygame.display.set_mode((800,600), pygame.OPENGL|pygame.DOUBLEBUF|pygame.NOFRAME) #pygame.FULLSCREEN|pygame.NOFRAME

vertex_shader="""
varying vec3 v;
varying vec3 N;
varying vec4 C;

void main(void)
{

v = gl_ModelViewMatrix * gl_Vertex;
N = gl_NormalMatrix * gl_Normal;
C = gl_Color;

gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;

}
""";

fragment_shader="""
varying vec3 N;
varying vec3 v;
varying vec4 C;
uniform float f;

void main(void)
{
vec3 L = gl_LightSource[0].position.xyz-v;

// "Lambert's law"? (see notes)
// Rather: faces will appear dimmer when struck in an acute angle
// distance attenuation

float Idiff = max(dot(normalize(L),N),0.0)*pow(length(L),-2.0);

gl_FragColor = C + f * vec4(1.0,1.0,1.0,1.0)*Idiff; // diffuse reflection
}
""";

# load a cube into a display list

box = Box([0.05,0.06,0.07])
shader = Shader(vertex_shader, fragment_shader)
rt = RenderTexture(128,128,GL_RGB,False)
last_time = pygame.time.get_ticks()
elapsed_time = 1000

while True:

    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(90,1,0.01,1000)
    gluLookAt(dist*sin(t/260.0)*4,dist*cos(t/260.0)*4,dist*cos(t/687.0)*3,0,0,0,0,1,0)

    if (pixelate):
        rt.enable()

    glClearColor(0.05, 0.06, 0.07, 1.0)
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)

    glMatrixMode(GL_MODELVIEW)

    # calculate light source position
    ld=[sin(t/16.0)*4.0,sin(t/20.0)*4.0,cos(t/16.0)*4.0]
    # pass data to fragment shader
    glLightfv(GL_LIGHT0,GL_POSITION,[ld[0],ld[1],ld[2]]);

    # fallback
    glColor3f(1,1,1)

    glLoadIdentity()
    # render a pretty range of cubes

    if(use_blending):
        glBlendFunc( GL_ONE, GL_ONE )
        glEnable( GL_BLEND )
        glDisable(GL_DEPTH_TEST)
        glEnable(GL_CULL_FACE)
    else:
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_CULL_FACE)
        
    glLineWidth(1)

    pygame.event.pump()
    key=pygame.key.get_pressed()

    shader.enable()
    shader.setUniformFloat("f",1.0)

    for i in range(-voxelsize,voxelsize):
        for j in range(-voxelsize,voxelsize):
            for k in range(-voxelsize,voxelsize):

                if (shape_sphere and distance([i,j,k],[0,0,0]) > voxelsize * 0.9):
                    continue

                glPushMatrix()

                glTranslate(i,j,k)
                if (rotate_cubes):
                    glRotate(t*i*0.1,0,1,0)
                    glRotate(t*j*0.1,1,0,0)
                    
                f = 0.5 # + 0.1 * abs( sin(i + j + k) )
                f = 1 / max(1.0, distance([i,j,k],ld))
                f= max(f*f,0.2)
                
                glScale(f,f,f)
                box.render()

                if render_lines:
                    glScale(1.1,1.1,1.1)
                    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
                    box.render()
                    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
                glPopMatrix()

    #final cube
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
    glPushMatrix()
    glScale(voxelsize-1,voxelsize-1,voxelsize-1)
    box.render()
    glPopMatrix()
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
                
    glDisable(GL_BLEND)
    
    shader.disable()    

    if (pixelate):
        rt.disable()
        rt.render()

    if key[pygame.K_SPACE]:
        dist *= 1.01
    if key[pygame.K_BACKSPACE]:
        dist *= 0.99

    renderFPSbar(elapsed_time)
                        
    pygame.display.flip()
    if (process_input(pygame.event.get()) == False):
        pygame.display.quit()
        sys.exit(0)

    current_time = pygame.time.get_ticks()
    elapsed_time = current_time - last_time
    last_time = current_time

    t += elapsed_time * 0.03



